@@ -1,8 +1,6 @@
-(import (only (gnu packages compression) gzip))
-
 (define (create-file filename)
   (make-process
-   (name (string-append "create-file-" (basename filename)))
+   (name "create-file")
    (outputs (list filename))
    (run-time (complexity
               (space   20 mebibytes)
@@ -14,8 +12,8 @@
 
 (define (compress-file input)
   (make-process
-   (name (string-append "compress-file-" (basename input)))
-   (packages (list gzip))
+   (name "compress-file")
+   (packages (list "gzip"))
    (inputs (list input))
    (outputs (list (string-append input ".gz")))
    (run-time (complexity
@@ -26,9 +24,9 @@
 (make-workflow
  (name "dynamic-workflow")
  (processes
-  (let* ((files '("/tmp/one.txt"
-                  "/tmp/two.txt"
-                  "/tmp/three.txt"))
+  (let* ((files '("one.txt"
+                  "two.txt"
+                  "three.txt"))
          (create-file-processes
           (map create-file files))
          (compress-file-processes
