@@ -1,4 +1,4 @@
-;;; Copyright © 2021 Ricardo Wurmus <rekado@elephly.net>
+;;; Copyright © 2021, 2022 Ricardo Wurmus <rekado@elephly.net>
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify it
 ;;; under the terms of the GNU General Public License as published by
@@ -15,6 +15,7 @@
 
 (define-module (guix extensions workflow)
   #:use-module (guix scripts)
+  #:use-module (ice-9 match)
   #:export (guix-workflow))
 
 ;; IMPORTANT: We must avoid loading any modules from the GWL here,
@@ -43,7 +44,7 @@
                   ;; Guile's own load path, so we put it at the very
                   ;; end.  We add it for all the additional Guile
                   ;; packages.
-                  (parse-path "/gnu/store/iv10yw8my6hpzczmlk9nr89j2prscrpg-profile/share/guile/site/3.0:/home/rekado/.guix-profile/share/guile/site/3.0:/run/current-system/profile/share/guile/site/3.0:/home/rekado/.guix-profile/share/guile/site/3.0:/run/current-system/profile/share/guile/site/3.0")))
+                  (parse-path "/gnu/store/n1fw735w2rznhw7kh926flipkbfhr1dh-profile/share/guile/site/3.0:/home/rekado/.guix-home/profile/share/guile/site/3.0:/home/rekado/.guix-profile/share/guile/site/3.0:/run/current-system/profile/share/guile/site/3.0")))
     (set! %load-compiled-path
           (append own-load-compiled-path
                   ;; This is Guile's own pristine load path for
@@ -59,7 +60,7 @@
                   ;; Guile's own load path, so we put it at the very
                   ;; end.  We add it for all the additional Guile
                   ;; packages.
-                  (parse-path "/gnu/store/iv10yw8my6hpzczmlk9nr89j2prscrpg-profile/lib/guile/3.0/site-ccache:/gnu/store/iv10yw8my6hpzczmlk9nr89j2prscrpg-profile/share/guile/site/3.0:/home/rekado/.guix-profile/lib/guile/3.0/site-ccache:/home/rekado/.guix-profile/share/guile/site/3.0:/run/current-system/profile/lib/guile/3.0/site-ccache:/run/current-system/profile/share/guile/site/3.0:/home/rekado/.guix-profile/lib/guile/3.0/site-ccache:/home/rekado/.guix-profile/share/guile/site/3.0:/run/current-system/profile/lib/guile/3.0/site-ccache:/run/current-system/profile/share/guile/site/3.0")))))
+                  (parse-path "/gnu/store/n1fw735w2rznhw7kh926flipkbfhr1dh-profile/lib/guile/3.0/site-ccache:/gnu/store/n1fw735w2rznhw7kh926flipkbfhr1dh-profile/share/guile/site/3.0:/home/rekado/.guix-home/profile/lib/guile/3.0/site-ccache:/home/rekado/.guix-home/profile/share/guile/site/3.0:/home/rekado/.guix-profile/lib/guile/3.0/site-ccache:/home/rekado/.guix-profile/share/guile/site/3.0:/run/current-system/profile/lib/guile/3.0/site-ccache:/run/current-system/profile/share/guile/site/3.0")))))
 
 ;;;
 ;;; Entry point.
@@ -70,7 +71,14 @@
   (synopsis "execute or visualize workflows")
 
   (replace-load-paths!)
-  (let ((main (module-ref (resolve-interface '(gwl main))
-                          'guix-workflow-main)))
-    ;;(bindtextdomain "guix-workflow" "/usr/local/share/locale")
-    (apply main args)))
+  (match (command-line)
+    ((guix workflow . args)
+     (setenv "GUILE_LOAD_PATH"
+             (string-join %load-path ":"))
+     (setenv "GUILE_LOAD_COMPILED_PATH"
+             (string-join %load-compiled-path ":"))
+     (setenv "_GWL_INVOKING_GUIX"
+             (dirname (dirname guix)))
+     (apply execl "/gnu/store/n1fw735w2rznhw7kh926flipkbfhr1dh-profile/bin/guile" "/gnu/store/n1fw735w2rznhw7kh926flipkbfhr1dh-profile/bin/guile" "-q"
+            "-c" "(apply (@ (gwl main) guix-workflow-main) (command-line))"
+            "--" args))))
