@@ -23,6 +23,8 @@
                 #:select (mkdir-p))
   #:use-module ((guix base32)
                 #:select (bytevector->base32-string))
+  #:use-module ((guix utils)
+                #:select (cache-directory))
   #:use-module ((rnrs bytevectors)
                 #:select (string->utf8
                           bytevector->u8-list
@@ -47,7 +49,11 @@
 
 ;;; Data cache
 
-(define %cache-root (make-parameter "/tmp/gwl"))
+(define %cache-root
+  (make-parameter
+   (string-append (cache-directory #:ensure? #false)
+                  "/workflows")))
+
 (define %cache-delay (make-parameter 0))
 
 (define (process->hash process scripts-table)
