@@ -31,7 +31,7 @@
   "GNU GWL")
 
 (define %gwl-version
-  "0.4.0")
+  "0.5.0")
 
 (define %gwl-bug-report-address
   "gwl-devel@gnu.org")
@@ -65,7 +65,6 @@ workflow.")
        (list
         (argument
          (name 'file)
-         (optional? #false)
          (test file-exists?)
          (handler identity)
          (example "/path/to/my-workflow.w"))))
@@ -95,12 +94,19 @@ workflow.")
          (character #\i)
          (synopsis "Specify workflow input NAME, optionally mapped to FILE")
          (example "NAME[=FILE]")
-         (test string?))
+         (default #false)
+         (test (match-lambda
+                 (#false #true)
+                 (value (string? value)))))
         (switch
          (name 'output)
          (character #\o)
          (synopsis "Set LOCATION as output for a workflow")
-         (example "LOCATION"))
+         (example "LOCATION")
+         (default #false)
+         (test (match-lambda
+                 (#false #true)
+                 (value (string? value)))))
         (switch
          (name 'engine)
          (character #\e)
@@ -147,11 +153,9 @@ format) of the specified workflow.")
        (list
         (argument
          (name 'file)
-         (optional? #false)
          (test file-exists?)
          (handler identity)
          (example "/path/to/my-workflow.w")))))
-
      (configuration
       (name 'web)
       (synopsis "Start the web interface")
@@ -234,7 +238,7 @@ servers.")
          (name 'dot)
          (synopsis "The file name of the 'dot' command")
          (character #false)
-         (default "/gnu/store/iv10yw8my6hpzczmlk9nr89j2prscrpg-profile/bin/dot")
+         (default "/gnu/store/n1fw735w2rznhw7kh926flipkbfhr1dh-profile/bin/dot")
          (example "/path/to/dot")
          (test (lambda (value)
                  (and (string? value)
@@ -248,9 +252,9 @@ servers.")
                  (and (string? value)
                       (file-exists? value))))))))))
    (generate-cmdtree? #false)
-   (version "0.4.0")
+   (version "0.5.0")
    (license gpl3+)
-   (copyright '(2016 2017 2018 2019 2020))
+   (copyright '(2016 2017 2018 2019 2020 2021 2022))
    (author "Guix Workflow Language authors")))
 
 ;; This is useful for REPL sessions where %config is not initialized
