@@ -38,7 +38,7 @@
   #:use-module (srfi srfi-26)
   #:use-module (srfi srfi-34)
   #:use-module (srfi srfi-35)
-  #:use-module (ice-9 popen)
+  #:use-module (ice-9 threads)
   #:use-module (ice-9 rdelim)
   #:export (make-process
             process?
@@ -371,7 +371,7 @@ of PROCESS."
     ("_GWL_PROCESS_COMPLEXITY_TIME" .
      ,(or (and=> (process-time process) number->string) ""))))
 
-(define-syntax-rule (snippet-caller code-converter command)
+(define (snippet-caller code-converter command)
   (lambda (process code)
     `(begin
        (use-modules (srfi srfi-1))
@@ -384,40 +384,40 @@ of PROCESS."
                                  value))))
                  ',(process->env process))
        (let ((retval (apply system*
-                            (append command
-                                    (code-converter (string-join (map (lambda (val)
-                                                                        (if (list? val)
-                                                                            (format #f "~{~a~^ ~}" val)
-                                                                            (format #f "~a" val)))
-                                                                      (list ,@code))
-                                                                 ""))))))
+                            (append ,command
+                                    (,code-converter (string-join (map (lambda (val)
+                                                                         (if (list? val)
+                                                                             (format #f "~{~a~^ ~}" val)
+                                                                             (format #f "~a" val)))
+                                                                       (list ,@code))
+                                                                  ""))))))
          (or (zero? retval) (exit retval))))))
 
 (define language-python
   (make <language>
     #:name 'python
-    #:call (snippet-caller list '("python3" "-c"))))
+    #:call (snippet-caller 'list '(list "python3" "-c"))))
 
 (define language-r
   (make <language>
     #:name 'R
     #:call (snippet-caller
-            (lambda (code)
-              (append-map (lambda (line)
-                            (list "-e" line))
-                          (filter (negate string-null?)
-                                  (string-split code #\newline))))
-            '("Rscript"))))
+            '(lambda (code)
+               (append-map (lambda (line)
+                             (list "-e" line))
+                           (filter (negate string-null?)
+                                   (string-split code #\newline))))
+            '(list "Rscript"))))
 
 (define language-bash
   (make <language>
     #:name 'bash
-    #:call (snippet-caller list '("bash" "-c"))))
+    #:call (snippet-caller 'list '(list "bash" "-c"))))
 
 (define language-sh
   (make <language>
     #:name 'sh
-    #:call (snippet-caller list '("/bin/sh" "-c"))))
+    #:call (snippet-caller 'list '(list "/bin/sh" "-c"))))
 
 (define languages
   (list language-sh
@@ -471,23 +471,63 @@ written to a file."
                              language-call)
                       ;; There is no pre-defined way to execute the
                       ;; snippet.  Use generic approach.
-                      (snippet-caller list
-                                      (list
-                                       (string-append (getenv "_GWL_PROFILE")
-                                                      #$(sanitize-path (symbol->string name))))))))
+                      (snippet-caller 'list
+                                      `(list
+                                        (string-append (getenv "_GWL_PROFILE")
+                                                       ,(sanitize-path (symbol->string name)))
+                                        ,@(code-snippet-arguments snippet))))))
        (call process code)))))
 
+(define* (call-with-output-processor command proc #:optional capture-stderr?)
+  "Silently execute COMMAND, a list of strings representing an
+executable with its arguments, and apply PROC to every line printed to
+standard output and, optionally when CAPTURE-STDERR? is #T, standard
+error.  Return the exit status of COMMAND."
+  ;; We can only capture a program's standard error by parameterizing
+  ;; current-error-port to a *file* port before using system* or
+  ;; open-pipe*.  The process will write its standard error stream to
+  ;; the provided file descriptor.  Meanwhile we read from the file
+  ;; descriptor (blocking) for new lines until the process exits.
+  (match (socketpair PF_UNIX SOCK_STREAM 0)
+    ((in . out)
+     (let ((err (if capture-stderr?
+                    (dup out)
+                    (%make-void-port "w"))))
+       (catch #true
+         (lambda ()
+           (let ((thread
+                  (parameterize ((current-error-port err)
+                                 (current-output-port out))
+                    (call-with-new-thread
+                     (lambda ()
+                       (let ((status
+                              (status:exit-val
+                               (apply system* command))))
+                         (close-port err)
+                         (close-port out)
+                         status))))))
+             (let loop ()
+               (match (read-line in 'concat)
+                 ((? eof-object?)
+                  (join-thread thread))
+                 (line
+                  (proc line)
+                  (loop))))))
+         (lambda (key . args)
+           (for-each
+            (lambda (port)
+              (false-if-exception (close-port port)))
+            (list err out in))
+           (apply throw key args)))))))
+
 (define (run-process-command command)
   "Run the COMMAND, a list consisting of an executable and its
-arguments."
-  (let ((port (apply open-pipe* OPEN_READ command)))
-    (let loop ()
-      (let ((line (read-line port 'concat)))
-        (if (eof-object? line)
-            (close-pipe port)
-            (begin
-              (log-event 'process line)
-              (loop)))))))
+arguments.  Log any output on the standard output or standard error
+streams as a 'process event."
+  (call-with-output-processor
+   command
+   (cut log-event 'process <>)
+   'capture-stderr))
 
 
 (define* (script-modules #:optional containerize?)
@@ -496,13 +536,14 @@ arguments."
         (gnu build linux-container)
         (gnu system file-systems)
         (guix build utils)
-        (guix search-paths))
+        (guix search-paths)
+        (ice-9 match))
       '((guix search-paths))))
 
-(define* (containerize script #:key inputs outputs)
-  "Call SCRIPT, a program-file object, in a G-expression that sets up
-a container where the provided INPUTS are available (in addition to
-all store items needed for execution).  OUTPUTS are copied outside of
+(define (containerize script)
+  "Call SCRIPT, a program-file object, in a G-expression that sets up a
+container where the provided inputs are available (in addition to all
+store items needed for execution).  All outputs are copied outside of
 the container."
   (with-imported-modules (source-module-closure
                           (script-modules 'container))
@@ -510,7 +551,8 @@ the container."
         (use-modules (gnu build accounts)
                      (gnu build linux-container)
                      (gnu system file-systems)
-                     (guix build utils))
+                     (guix build utils)
+                     (ice-9 match))
         (define (location->file-system source target writable?)
           (file-system
             (device source)
@@ -533,44 +575,48 @@ the container."
                (groups (list (group-entry (name "users") (gid gid))
                              (group-entry (gid 65534) ;the overflow GID
                                           (name "overflow")))))
-          (call-with-container
-              (append %container-file-systems
-                      ;; Current directory for final outputs
-                      (list (location->file-system
-                             (canonicalize-path ".") "/gwl" #t))
-                      (map (lambda (location)
-                             (location->file-system location location #f))
-                           (append (call-with-input-file #$(references-file script) read)
-                                   '#$inputs)))
-            (lambda ()
-              (unless (file-exists? "/bin/sh")
-                (unless (file-exists? "/bin")
-                  (mkdir "/bin"))
-                (symlink #$(file-append (bash-minimal) "/bin/sh") "/bin/sh"))
+          (match (command-line)
+            ((_ (= (lambda (s) (call-with-input-string s read))
+                   #{ %gwl process-arguments}#) . rest)
+             (call-with-container
+                 (append %container-file-systems
+                     ;; Current directory for final outputs
+                     (list (location->file-system
+                            (canonicalize-path ".") "/gwl" #t))
+                   (map (lambda (location)
+                          (location->file-system location location #f))
+                        (append (call-with-input-file #$(references-file script) read)
+                            (assoc-ref #{ %gwl process-arguments}# 'inputs))))
+               (lambda ()
+                 (unless (file-exists? "/bin/sh")
+                   (unless (file-exists? "/bin")
+                     (mkdir "/bin"))
+                   (symlink #$(file-append (bash-minimal) "/bin/sh") "/bin/sh"))
 
-              ;; Setup directory for temporary files.
-              (mkdir-p "/tmp")
-              (for-each (lambda (var)
-                          (setenv var "/tmp"))
-                        '("TMPDIR" "TEMPDIR"))
+                 ;; Setup directory for temporary files.
+                 (mkdir-p "/tmp")
+                 (for-each (lambda (var)
+                             (setenv var "/tmp"))
+                           '("TMPDIR" "TEMPDIR"))
 
-              ;; Create a dummy /etc/passwd to satisfy applications that demand
-              ;; to read it.
-              (unless (file-exists? "/etc")
-                (mkdir "/etc"))
-              (write-passwd (list passwd))
-              (write-group groups)
+                 ;; Create a dummy /etc/passwd to satisfy applications that demand
+                 ;; to read it.
+                 (unless (file-exists? "/etc")
+                   (mkdir "/etc"))
+                 (write-passwd (list passwd))
+                 (write-group groups)
 
-              (system* #$script)
+                 (apply system* #$script (cdr (command-line)))
 
-              ;; Copy generated files to final directory.
-              (for-each (lambda (output)
-                          (let ((target (string-append "/gwl/" output)))
-                            (mkdir-p (dirname target))
-                            (copy-file output target)))
-                        (filter file-exists? '#$outputs)))
-            #:guest-uid uid
-            #:guest-gid gid)))))
+                 ;; Copy generated files to final directory.
+                 (for-each (lambda (output)
+                             (let ((target (string-append "/gwl/" output)))
+                               (mkdir-p (dirname target))
+                               (copy-file output target)))
+                           (filter file-exists?
+                                   (assoc-ref #{ %gwl process-arguments}# 'outputs))))
+               #:guest-uid uid
+               #:guest-gid gid)))))))
 
 ;;; ---------------------------------------------------------------------------
 ;;; ADDITIONAL FUNCTIONS
@@ -642,7 +688,10 @@ PROCESS."
   (let* ((name         (process-full-name process))
          (packages     (cons (bash-minimal)
                              (process-packages process)))
-         (manifest     (packages->manifest packages))
+         (manifest     (packages->manifest (map
+                                            (lambda (pkg)
+                                              (list pkg (package-output pkg)))
+                                            packages)))
          (profile      (profile (content manifest)))
          (search-paths (delete-duplicates
                         (map search-path-specification->sexp
@@ -671,15 +720,7 @@ PROCESS."
          (computed-script
           (if containerize?
               (program-file (string-append "gwl-" name "-container.scm")
-                            (containerize script
-                                          #:inputs
-                                          (append
-                                           ;; Data inputs
-                                           (process-inputs process)
-                                           ;; Files mapped to free inputs
-                                           input-files)
-                                          #:outputs
-                                          (process-outputs process))
+                            (containerize script)
                             #:guile (default-guile))
               script)))
     computed-script))
