@@ -23,6 +23,8 @@
   #:use-module (ice-9 match)
   #:use-module (language wisp)
   #:use-module (system base language)
+  #:use-module ((system base compile)
+                #:select (default-optimization-level))
 
   #:use-module (gwl config)
   #:use-module (gwl errors)
@@ -41,6 +43,10 @@
                           derivation-output-path))
   #:use-module ((guix ui)
                 #:select (build-notifier))
+  #:use-module ((guix scripts package)
+                #:select (%package-default-options))
+  #:use-module ((guix scripts build)
+                #:select (set-build-options-from-command-line))
   #:use-module ((guix status)
                 #:select (with-status-verbosity))
 
@@ -137,22 +143,8 @@
         ((string-suffix? ".gwl" file) ".gwl")
         (else #f)))
 
-(define wisp-reader
-  ;; XXX We'd like to use language-reader here, but (language wisp
-  ;; spec) triggers a very annoying setlocale warning because it
-  ;; evaluates (setlocale LC_ALL "foo").
-  #;
-  (language-reader (lookup-language 'wisp))
-  (lambda (port env)
-    ;; allow using "# foo" as #(foo).
-    (read-hash-extend #\# (λ (chr port) #\#))
-    (cond
-     ((eof-object? (peek-char port))
-      (read-char port)) ; return eof: we’re done
-     (else
-      (match (wisp-scheme-read-chunk port)
-        (() #false)
-        ((chunk . _) chunk))))))
+(define wisp (lookup-language 'wisp))
+(define wisp-reader (language-reader wisp))
 
 
 ;;; Workflow profile and environment
@@ -191,15 +183,19 @@ modify the load path of the current process."
                        (map lookup-package package-names)))
        (profile       (profile (content manifest)))
        (profile-directory
-        (with-status-verbosity (%config 'verbosity)
-          (with-build-handler (build-notifier #:verbosity (%config 'verbosity))
-            (run-with-store (inferior-store)
-              (mlet* %store-monad
-                  ((drv (lower-object profile))
-                   (_ (built-derivations (list drv))))
-                (match (derivation-outputs drv)
-                  (((_ . output) . rest)
-                   (return (derivation-output-path output))))))))))
+        (parameterize ((%guile-for-build (default-guile-derivation)))
+          (log-event 'info (G_ "Building workflow profile.~%"))
+          (with-status-verbosity (%config 'verbosity)
+            (set-build-options-from-command-line
+             (inferior-store) %package-default-options)
+            (with-build-handler (build-notifier #:verbosity (%config 'verbosity))
+              (run-with-store (inferior-store)
+                (mlet* %store-monad
+                    ((drv (lower-object profile))
+                     (_ (built-derivations (list drv))))
+                  (match (derivation-outputs drv)
+                    (((_ . output) . rest)
+                     (return (derivation-output-path output)))))))))))
     (for-each (match-lambda
                 ((($ <search-path-specification>
                      "GUILE_LOAD_PATH" _ separator) . value)
@@ -298,8 +294,11 @@ where all the basic GWL modules are available."
        (lambda ()
          (set-current-module user-module)
 
-         ;; Hide the "auto-compiling" messages.
-         (parameterize ((current-warning-port (%make-void-port "w")))
+         (parameterize ((default-optimization-level 0) ; make compilation fast
+                        (current-language (if (wisp-suffix file)
+                                              wisp (current-language)))
+                        ;; Hide the "auto-compiling" messages.
+                        (current-warning-port (%make-void-port "w")))
            (call-with-prompt tag
              (lambda ()
                ;; XXX: The Wisp reader fails to set source properties in all
@@ -308,10 +307,7 @@ where all the basic GWL modules are available."
                                (make-parameter file))
                ;; Give 'load' an absolute file name so that it doesn't
                ;; try to search for FILE in %LOAD-COMPILED-PATH.
-               (load (canonicalize-path file)
-                     (and (wisp-suffix file)
-                          (lambda (port)
-                            (wisp-reader port user-module)))))
+               (load (canonicalize-path file)))
              (const #f))))))
     (lambda _
       (exit 1))
